@@ -16,6 +16,8 @@
 
 source $BASE/new/devstack/openrc admin admin
 
+set -e
+
 function generate_testr_results {
     if [ -f .testrepository/0 ]; then
         sudo .tox/py27-gate/bin/testr last --subunit > $WORKSPACE/testrepository.subunit
@@ -34,14 +36,13 @@ export GNOCCHI_DIR="$BASE/new/gnocchi"
 sudo chown -R stack:stack $GNOCCHI_DIR
 cd $GNOCCHI_DIR
 
-keystone endpoint-list
-keystone service-list
-keystone endpoint-get --service metric
+openstack catalog list
 
-curl -X GET http://localhost:8041/v1/archive_policy -H "Content-Type: application/json"
+export GNOCCHI_SERVICE_TOKEN=$(openstack token issue -c id -f value)
+export GNOCCHI_SERVICE_URL=$(openstack catalog show metric -c endpoints -f value | awk '/publicURL/{print $2}')
+
+curl -X GET ${GNOCCHI_SERVICE_URL}/v1/archive_policy -H "Content-Type: application/json"
 
-export GNOCCHI_SERVICE_HOST=localhost
-export GNOCCHI_SERVICE_PORT=8041
 
 # Run tests
 echo "Running gnocchi functional test suite"
