@@ -20,6 +20,7 @@ import uuid
 from gnocchi import archive_policy
 from gnocchi import indexer
 from gnocchi.tests import base as tests_base
+from gnocchi import utils
 
 
 class TestIndexer(tests_base.TestCase):
@@ -86,7 +87,7 @@ class TestIndexerDriver(tests_base.TestCase):
         user = uuid.uuid4()
         project = uuid.uuid4()
         rc = self.index.create_resource('generic', r1, user, project)
-        self.assertIsNotNone(rc['started_at'])
+        self.assertIsNotNone(rc.started_at)
         self.assertIsNotNone(rc.revision_start)
         self.assertEqual({"id": r1,
                           "revision_start": rc.revision_start,
@@ -131,11 +132,11 @@ class TestIndexerDriver(tests_base.TestCase):
         kwargs = {'server_group': server_group} if server_group else {}
 
         rc = self.index.create_resource('instance', r1, user, project,
-                                        flavor_id=1,
+                                        flavor_id="1",
                                         image_ref="http://foo/bar",
                                         host="foo",
                                         display_name="lol", **kwargs)
-        self.assertIsNotNone(rc['started_at'])
+        self.assertIsNotNone(rc.started_at)
         self.assertIsNotNone(rc.revision_start)
         self.assertEqual({"id": r1,
                           "revision_start": rc.revision_start,
@@ -151,13 +152,13 @@ class TestIndexerDriver(tests_base.TestCase):
                           "server_group": server_group,
                           "host": "foo",
                           "image_ref": "http://foo/bar",
-                          "flavor_id": 1,
+                          "flavor_id": "1",
                           "metrics": {}},
                          rc.jsonify())
         rg = self.index.get_resource('generic', r1, with_metrics=True)
-        self.assertEqual(rc['id'], rg['id'])
-        self.assertEqual(rc["revision_start"], rg["revision_start"])
-        self.assertEqual(rc['metrics'], rg['metrics'])
+        self.assertEqual(rc.id, rg.id)
+        self.assertEqual(rc.revision_start, rg.revision_start)
+        self.assertEqual(rc.metrics, rg.metrics)
 
     def test_create_instance(self):
         self._do_test_create_instance()
@@ -191,7 +192,7 @@ class TestIndexerDriver(tests_base.TestCase):
 
     def test_create_resource_with_start_timestamp(self):
         r1 = uuid.uuid4()
-        ts = datetime.datetime(2014, 1, 1, 23, 34, 23, 1234)
+        ts = utils.datetime_utc(2014, 1, 1, 23, 34, 23, 1234)
         user = uuid.uuid4()
         project = uuid.uuid4()
         rc = self.index.create_resource(
@@ -226,7 +227,7 @@ class TestIndexerDriver(tests_base.TestCase):
                                  archive_policy_name="low")
         rc = self.index.create_resource('generic', r1, user, project,
                                         metrics={'foo': e1, 'bar': e2})
-        self.assertIsNotNone(rc['started_at'])
+        self.assertIsNotNone(rc.started_at)
         self.assertIsNotNone(rc.revision_start)
         self.assertEqual({"id": r1,
                           "revision_start": rc.revision_start,
@@ -241,7 +242,7 @@ class TestIndexerDriver(tests_base.TestCase):
                           "metrics": {'foo': str(e1), 'bar': str(e2)}},
                          rc.jsonify())
         r = self.index.get_resource('generic', r1, with_metrics=True)
-        self.assertIsNotNone(r['started_at'])
+        self.assertIsNotNone(r.started_at)
         self.assertEqual({"id": r1,
                           "revision_start": r.revision_start,
                           "revision_end": None,
@@ -272,7 +273,7 @@ class TestIndexerDriver(tests_base.TestCase):
         self.index.update_resource(
             'generic',
             r1,
-            ended_at=datetime.datetime(2043, 1, 1, 2, 3, 4))
+            ended_at=utils.datetime_utc(2043, 1, 1, 2, 3, 4))
         r = self.index.get_resource('generic', r1, with_metrics=True)
         self.assertIsNotNone(r.started_at)
         self.assertIsNone(r.user_id)
@@ -282,7 +283,7 @@ class TestIndexerDriver(tests_base.TestCase):
         self.assertEqual(r1, r.id)
         self.assertEqual(user, r.created_by_user_id)
         self.assertEqual(project, r.created_by_project_id)
-        self.assertEqual(datetime.datetime(2043, 1, 1, 2, 3, 4), r.ended_at)
+        self.assertEqual(utils.datetime_utc(2043, 1, 1, 2, 3, 4), r.ended_at)
         self.assertEqual("generic", r.type)
         self.assertEqual(0, len(r.metrics))
         self.index.update_resource(
@@ -290,10 +291,10 @@ class TestIndexerDriver(tests_base.TestCase):
             r1,
             ended_at=None)
         r = self.index.get_resource('generic', r1, with_metrics=True)
-        self.assertIsNotNone(r['started_at'])
-        self.assertIsNotNone(r["revision_start"])
+        self.assertIsNotNone(r.started_at)
+        self.assertIsNotNone(r.revision_start)
         self.assertEqual({"id": r1,
-                          "revision_start": r["revision_start"],
+                          "revision_start": r.revision_start,
                           "revision_end": None,
                           "ended_at": None,
                           "created_by_user_id": user,
@@ -365,7 +366,7 @@ class TestIndexerDriver(tests_base.TestCase):
         user = uuid.uuid4()
         project = uuid.uuid4()
         rc = self.index.create_resource('instance', r1, user, project,
-                                        flavor_id=1,
+                                        flavor_id="1",
                                         image_ref="http://foo/bar",
                                         host="foo",
                                         display_name="lol")
@@ -373,10 +374,25 @@ class TestIndexerDriver(tests_base.TestCase):
         r = self.index.get_resource('instance', r1, with_metrics=True)
         self.assertEqual(rc, r)
 
+    def test_update_resource_ended_at_fail(self):
+        r1 = uuid.uuid4()
+        user = uuid.uuid4()
+        project = uuid.uuid4()
+        self.index.create_resource('instance', r1, user, project,
+                                   flavor_id="1",
+                                   image_ref="http://foo/bar",
+                                   host="foo",
+                                   display_name="lol")
+        self.assertRaises(
+            indexer.ResourceValueError,
+            self.index.update_resource,
+            'instance', r1,
+            ended_at=utils.datetime_utc(2010, 1, 1, 1, 1, 1))
+
     def test_update_resource_unknown_attribute(self):
         r1 = uuid.uuid4()
         self.index.create_resource('instance', r1, uuid.uuid4(), uuid.uuid4(),
-                                   flavor_id=1,
+                                   flavor_id="1",
                                    image_ref="http://foo/bar",
                                    host="foo",
                                    display_name="lol")
@@ -427,7 +443,7 @@ class TestIndexerDriver(tests_base.TestCase):
                                         metrics={'foo': e1, 'bar': e2})
         self.index.delete_metric(e1)
         r = self.index.get_resource('generic', r1, with_metrics=True)
-        self.assertIsNotNone(r['started_at'])
+        self.assertIsNotNone(r.started_at)
         self.assertIsNotNone(r.revision_start)
         self.assertEqual({"id": r1,
                           "started_at": r.started_at,
@@ -445,7 +461,7 @@ class TestIndexerDriver(tests_base.TestCase):
         r1 = uuid.uuid4()
         created = self.index.create_resource('instance', r1,
                                              uuid.uuid4(), uuid.uuid4(),
-                                             flavor_id=123,
+                                             flavor_id="123",
                                              image_ref="foo",
                                              host="dwq",
                                              display_name="foobar")
@@ -502,7 +518,7 @@ class TestIndexerDriver(tests_base.TestCase):
         i = self.index.create_resource('instance', r2,
                                        user, project,
                                        user, project,
-                                       flavor_id=123,
+                                       flavor_id="123",
                                        image_ref="foo",
                                        host="dwq",
                                        display_name="foobar")
@@ -538,8 +554,8 @@ class TestIndexerDriver(tests_base.TestCase):
         g = self.index.create_resource(
             'generic', r1, user, project,
             user_id=user, project_id=project,
-            started_at=datetime.datetime(2010, 1, 1, 12, 0),
-            ended_at=datetime.datetime(2010, 1, 1, 13, 0))
+            started_at=utils.datetime_utc(2010, 1, 1, 12, 0),
+            ended_at=utils.datetime_utc(2010, 1, 1, 13, 0))
         resources = self.index.list_resources(
             'generic',
             attribute_filter={"and": [
@@ -566,7 +582,7 @@ class TestIndexerDriver(tests_base.TestCase):
         r2 = uuid.uuid4()
         i = self.index.create_resource('instance', r2,
                                        uuid.uuid4(), uuid.uuid4(),
-                                       flavor_id=123,
+                                       flavor_id="123",
                                        image_ref="foo",
                                        host="dwq",
                                        display_name="foobar")
@@ -575,10 +591,10 @@ class TestIndexerDriver(tests_base.TestCase):
         g_found = False
         i_found = False
         for r in resources:
-            if r['id'] == r1:
+            if r.id == r1:
                 self.assertEqual(g, r)
                 g_found = True
-            elif r['id'] == r2:
+            elif r.id == r2:
                 i_found = True
             if i_found and g_found:
                 break
@@ -588,7 +604,7 @@ class TestIndexerDriver(tests_base.TestCase):
         resources = self.index.list_resources('instance')
         self.assertGreaterEqual(len(resources), 1)
         for r in resources:
-            if r['id'] == r2:
+            if r.id == r2:
                 self.assertEqual(i, r)
                 break
         else:
@@ -619,7 +635,7 @@ class TestIndexerDriver(tests_base.TestCase):
                                               details=True)
         self.assertGreaterEqual(len(resources), 1)
         expected_resources = [r.jsonify() for r in resources
-                              if r['id'] == rid]
+                              if r.id == rid]
         self.assertIn(r2, expected_resources)
 
     def test_list_resources_with_history(self):
@@ -646,7 +662,7 @@ class TestIndexerDriver(tests_base.TestCase):
                                         project_id=new_project,
                                         append_metrics=True).jsonify()
 
-        r1['revision_end'] = r2["revision_start"]
+        r1['revision_end'] = r2['revision_start']
         r2['revision_end'] = None
         self.assertEqual({'foo': str(e1),
                           'bar': str(e2)}, r2['metrics'])
@@ -680,7 +696,7 @@ class TestIndexerDriver(tests_base.TestCase):
 
         r1 = self.index.create_resource('instance', rid, user, project,
                                         user, project,
-                                        flavor_id=123,
+                                        flavor_id="123",
                                         image_ref="foo",
                                         host="dwq",
                                         display_name="foobar_history",
@@ -691,7 +707,7 @@ class TestIndexerDriver(tests_base.TestCase):
                                         host="other",
                                         append_metrics=True).jsonify()
 
-        r1['revision_end'] = r2["revision_start"]
+        r1['revision_end'] = r2['revision_start']
         r2['revision_end'] = None
         self.assertEqual({'foo': str(e1),
                           'bar': str(e2)}, r2['metrics'])
@@ -717,33 +733,33 @@ class TestIndexerDriver(tests_base.TestCase):
         project = uuid.uuid4()
         g = self.index.create_resource(
             'generic', r1, user, project,
-            started_at=datetime.datetime(2000, 1, 1, 23, 23, 23),
-            ended_at=datetime.datetime(2000, 1, 3, 23, 23, 23))
+            started_at=utils.datetime_utc(2000, 1, 1, 23, 23, 23),
+            ended_at=utils.datetime_utc(2000, 1, 3, 23, 23, 23))
         r2 = uuid.uuid4()
         i = self.index.create_resource(
             'instance', r2, user, project,
-            flavor_id=123,
+            flavor_id="123",
             image_ref="foo",
             host="dwq",
             display_name="foobar",
-            started_at=datetime.datetime(2000, 1, 1, 23, 23, 23),
-            ended_at=datetime.datetime(2000, 1, 4, 23, 23, 23))
+            started_at=utils.datetime_utc(2000, 1, 1, 23, 23, 23),
+            ended_at=utils.datetime_utc(2000, 1, 4, 23, 23, 23))
         resources = self.index.list_resources(
             'generic',
             attribute_filter={
                 "and":
                 [{">=": {"started_at":
-                         datetime.datetime(2000, 1, 1, 23, 23, 23)}},
+                         utils.datetime_utc(2000, 1, 1, 23, 23, 23)}},
                  {"<": {"ended_at":
-                        datetime.datetime(2000, 1, 5, 23, 23, 23)}}]})
+                        utils.datetime_utc(2000, 1, 5, 23, 23, 23)}}]})
         self.assertGreaterEqual(len(resources), 2)
         g_found = False
         i_found = False
         for r in resources:
-            if r['id'] == r1:
+            if r.id == r1:
                 self.assertEqual(g, r)
                 g_found = True
-            elif r['id'] == r2:
+            elif r.id == r2:
                 i_found = True
             if i_found and g_found:
                 break
@@ -759,7 +775,7 @@ class TestIndexerDriver(tests_base.TestCase):
             })
         self.assertGreaterEqual(len(resources), 1)
         for r in resources:
-            if r['id'] == r2:
+            if r.id == r2:
                 self.assertEqual(i, r)
                 break
         else:
